@@ -26,6 +26,7 @@
 #include <kipc.h>
 #include <kmessagebox.h>
 #include <knuminput.h>
+#include <kprocio.h>
 #include <ksimpleconfig.h>
 #include <kstandarddirs.h>
 #include <stdlib.h>
@@ -137,7 +138,7 @@ FontUseItem::FontUseItem(
 {
   KAcceleratorManager::setNoAccel( this );
   setTitle( name );
-  readFont();
+  readFont( false );
 }
 
 void FontUseItem::setDefault()
@@ -145,7 +146,7 @@ void FontUseItem::setDefault()
   setFont( _default, isFixedOnly() );
 }
 
-void FontUseItem::readFont()
+void FontUseItem::readFont( bool useDefaults )
 {
   KConfigBase *config;
 
@@ -158,6 +159,8 @@ void FontUseItem::readFont()
     deleteme = true;
   }
 
+  config->setReadDefaults( useDefaults );
+
   config->setGroup(_rcgroup);
   QFont tmpFnt(_default);
   setFont( config->readFontEntry(_rckey, &tmpFnt), isFixedOnly() );
@@ -239,7 +242,7 @@ FontAASettings::FontAASettings(QWidget *parent)
        " have a linear ordering of RGB sub-pixel, some have BGR."));
 
   for(int t=KXftConfig::SubPixel::None+1; t<=KXftConfig::SubPixel::Vbgr; ++t)
-    subPixelType->insertItem(aaPixmaps[t-1], i18n(KXftConfig::description((KXftConfig::SubPixel::Type)t).utf8()));
+    subPixelType->insertItem(aaPixmaps[t-1], KXftConfig::description((KXftConfig::SubPixel::Type)t));
 
 #ifdef HAVE_FONTCONFIG
   QLabel *hintingLabel=new QLabel(i18n("Hinting style: "), mw);
@@ -247,7 +250,7 @@ FontAASettings::FontAASettings(QWidget *parent)
   hintingStyle=new QComboBox(false, mw);
   layout->addMultiCellWidget(hintingStyle, 2, 2, 1, 3);
   for(int s=KXftConfig::Hint::NotSet+1; s<=KXftConfig::Hint::Full; ++s)
-    hintingStyle->insertItem(i18n(KXftConfig::description((KXftConfig::Hint::Style)s).utf8()));
+    hintingStyle->insertItem(KXftConfig::description((KXftConfig::Hint::Style)s));
 
   QString hintingText(i18n("Hinting is a process used to enhance the quality of fonts at small sizes."));
   QWhatsThis::add(hintingStyle, hintingText);
@@ -268,13 +271,16 @@ FontAASettings::FontAASettings(QWidget *parent)
 }
 
 bool FontAASettings::load()
+{
+  return load( false );
+}
+
+
+bool FontAASettings::load( bool useDefaults )
 {
   double     from, to;
-#ifdef HAVE_FONTCONFIG
-  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType|KXftConfig::HintStyle);
-#else
-  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
-#endif
+  KXftConfig xft(KXftConfig::constStyleSettings);
+
 
   if(xft.getExcludeRange(from, to))
      excludeRange->setChecked(true);
@@ -312,6 +318,8 @@ bool FontAASettings::load()
   {
     KConfig kglobals("kdeglobals", false, false);
 
+    kglobals.setReadDefaults( useDefaults );
+
     kglobals.setGroup("General");
     hStyle=KXftConfig::Hint::Medium;
     xft.setHintStyle(hStyle);
@@ -331,12 +339,9 @@ bool FontAASettings::load()
 
 bool FontAASettings::save( bool useAA )
 {
-#ifdef HAVE_FONTCONFIG
-  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType|KXftConfig::HintStyle);
-#else
-  KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
-#endif
+  KXftConfig xft(KXftConfig::constStyleSettings);
   KConfig    kglobals("kdeglobals", false, false);
+
   kglobals.setGroup("General");
 
   xft.setAntiAliasing( useAA );
@@ -346,18 +351,13 @@ bool FontAASettings::save( bool useAA )
   else
     xft.setExcludeRange(0, 0);
 
-  if(useSubPixel->isChecked())
-  {
-    KXftConfig::SubPixel::Type spType(getSubPixelType());
+  KXftConfig::SubPixel::Type spType(useSubPixel->isChecked()
+                                        ? getSubPixelType()
+                                        : KXftConfig::SubPixel::None);
 
-    xft.setSubPixelType(spType);
-    kglobals.writeEntry("XftSubPixel", KXftConfig::toStr(spType));
-  }
-  else
-  {
-    xft.setSubPixelType(KXftConfig::SubPixel::None);
-    kglobals.writeEntry("XftSubPixel", "");
-  }
+  xft.setSubPixelType(spType);
+  kglobals.writeEntry("XftSubPixel", KXftConfig::toStr(spType));
+  kglobals.writeEntry("XftAntialias", useAA);
 
   bool mod=false;
 #ifdef HAVE_FONTCONFIG
@@ -385,14 +385,7 @@ bool FontAASettings::save( bool useAA )
 
 void FontAASettings::defaults()
 {
-  excludeRange->setChecked(true);
-  excludeFrom->setValue(8.0);
-  excludeTo->setValue(15.0);
-  useSubPixel->setChecked(false);
-#ifdef HAVE_FONTCONFIG
-  hintingStyle->setCurrentItem(getIndex(KXftConfig::Hint::Medium));
-#endif
-  enableWidgets();
+  load( true );
 }
 
 int FontAASettings::getIndex(KXftConfig::SubPixel::Type spType)
@@ -401,7 +394,7 @@ int FontAASettings::getIndex(KXftConfig::SubPixel::Type spType)
   int index;
 
   for(index=0; index<subPixelType->count(); ++index)
-    if(subPixelType->text(index)==i18n(KXftConfig::description(spType).utf8()))
+    if(subPixelType->text(index)==KXftConfig::description(spType))
     {
       pos=index;
       break;
@@ -415,7 +408,7 @@ KXftConfig::SubPixel::Type FontAASettings::getSubPixelType()
   int t;
 
   for(t=KXftConfig::SubPixel::None; t<=KXftConfig::SubPixel::Vbgr; ++t)
-    if(subPixelType->currentText()==i18n(KXftConfig::description((KXftConfig::SubPixel::Type)t).utf8()))
+    if(subPixelType->currentText()==KXftConfig::description((KXftConfig::SubPixel::Type)t))
       return (KXftConfig::SubPixel::Type)t;
 
   return KXftConfig::SubPixel::None;
@@ -428,7 +421,7 @@ int FontAASettings::getIndex(KXftConfig::Hint::Style hStyle)
   int index;
 
   for(index=0; index<hintingStyle->count(); ++index)
-    if(hintingStyle->text(index)==i18n(KXftConfig::description(hStyle).utf8()))
+    if(hintingStyle->text(index)==KXftConfig::description(hStyle))
     {
       pos=index;
       break;
@@ -443,7 +436,7 @@ KXftConfig::Hint::Style FontAASettings::getHintStyle()
   int s;
 
   for(s=KXftConfig::Hint::NotSet; s<=KXftConfig::Hint::Full; ++s)
-    if(hintingStyle->currentText()==i18n(KXftConfig::description((KXftConfig::Hint::Style)s).utf8()))
+    if(hintingStyle->currentText()==KXftConfig::description((KXftConfig::Hint::Style)s))
       return (KXftConfig::Hint::Style)s;
 
   return KXftConfig::Hint::Medium;
@@ -602,20 +595,46 @@ KFonts::KFonts(QWidget *parent, const char *name, const QStringList &)
 
    lay = new QHBoxLayout(layout, KDialog::spacingHint());
    cbAA = new QCheckBox( i18n( "Use a&nti-aliasing for fonts" ), this);
+   cbAA->setTristate( true );
    QWhatsThis::add(cbAA, i18n("If this option is selected, KDE will smooth the edges of curves in "
                               "fonts."));
-   lay->addStretch();
-   QPushButton *aaSettingsButton = new QPushButton( i18n( "Configure..." ), this);
+   aaSettingsButton = new QPushButton( i18n( "Configure..." ), this);
    connect(aaSettingsButton, SIGNAL(clicked()), SLOT(slotCfgAa()));
-   connect(cbAA, SIGNAL(toggled(bool)), aaSettingsButton, SLOT(setEnabled(bool)));
    lay->addWidget( cbAA );
    lay->addWidget( aaSettingsButton );
+   lay->addStretch();
+
+   lay = new QHBoxLayout( layout, KDialog::spacingHint());
+   cbForceDpi = new QCheckBox( i18n( "Force fonts DPI" ), this );
+   lay->addWidget( cbForceDpi );
+   comboForceDpi = new QComboBox( this );
+   comboForceDpi->insertItem( i18n( "96 DPI" ));
+   comboForceDpi->insertItem( i18n( "120 DPI" ));
+   comboForceDpi->setDisabled( true );
+   QString whatsthis = i18n(
+       "<p>This option forces a specific DPI value for fonts. It may be useful"
+       " when the real DPI of the hardware is not detected properly and it"
+       " is also often misused when poor quality fonts are used that do not"
+       " look well with DPI values other than 96 or 120 DPI.</p>"
+       "<p>The use of this option is generally discouraged. For selecting proper DPI"
+       " value a better option is explicitly configuring it for the whole X server if"
+       " possible (e.g. DisplaySize in xorg.conf or adding <i>-dpi value</i> to"
+       " ServerLocalArgs= in $KDEDIR/share/config/kdm/kdmrc). When fonts do not render"
+       " properly with real DPI value better fonts should be used or configuration"
+       " of font hinting should be checked.</p>" );
+   QWhatsThis::add(cbForceDpi, whatsthis);
+   QWhatsThis::add(comboForceDpi, whatsthis);
+   connect( cbForceDpi, SIGNAL( toggled( bool )), comboForceDpi, SLOT( setEnabled( bool )));
+   connect( cbForceDpi, SIGNAL( toggled( bool )), SLOT( changed()));
+   connect( comboForceDpi, SIGNAL( activated( int )), SLOT( changed()));
+   lay->addWidget( comboForceDpi );
+   lay->addStretch();
 
    layout->addStretch(1);
 
    aaSettings=new FontAASettings(this);
 
-   connect(cbAA, SIGNAL(clicked()), SLOT(slotUseAntiAliasing()));
+   connect(cbAA, SIGNAL(stateChanged(int)), SLOT(slotUseAntiAliasing()));
 
    load();
 }
@@ -633,25 +652,37 @@ void KFonts::fontSelected()
 
 void KFonts::defaults()
 {
-  for ( int i = 0; i < (int) fontUseList.count(); i++ )
-    fontUseList.at( i )->setDefault();
-
-  useAA = true;
-  cbAA->setChecked(useAA);
+  load( true );
   aaSettings->defaults();
-  emit changed(true);
 }
 
 void KFonts::load()
+{
+  load( false );
+}
+
+
+void KFonts::load( bool useDefaults )
 {
   for ( uint i = 0; i < fontUseList.count(); i++ )
-    fontUseList.at( i )->readFont();
+    fontUseList.at( i )->readFont( useDefaults );
 
-  useAA_original = useAA = aaSettings->load();
-  kdDebug(1208) << "AA:" << useAA << endl;
-  cbAA->setChecked(useAA);
+  useAA_original = useAA = aaSettings->load( useDefaults ) ? QButton::On : QButton::Off;
+  cbAA->setChecked(useAA == QButton::On);
 
-  emit changed(false);
+  KConfig cfgfonts("kcmfonts", true);
+  cfgfonts.setGroup("General");
+  int dpi = cfgfonts.readNumEntry( "forceFontDPI", 0 );
+  cbForceDpi->setChecked( dpi == 96 || dpi == 120 );
+  comboForceDpi->setCurrentItem( dpi == 120 ? 1 : 0 );
+  dpi_original = dpi;
+  if( cfgfonts.readBoolEntry( "dontChangeAASettings", true )) {
+      cbAA->setNoChange();
+      useAA_original = useAA = QButton::NoChange;
+  }
+  aaSettingsButton->setEnabled( cbAA->state() == QButton::On );
+
+  emit changed( useDefaults );
 }
 
 void KFonts::save()
@@ -659,9 +690,28 @@ void KFonts::save()
 
   for ( FontUseItem* i = fontUseList.first(); i; i = fontUseList.next() )
       i->writeFont();
-
   KGlobal::config()->sync();
 
+  KConfig cfgfonts("kcmfonts");
+  cfgfonts.setGroup("General");
+  int dpi;
+  if( !cbForceDpi->isChecked())
+      dpi = 0;
+  else
+      dpi = comboForceDpi->currentItem() == 0 ? 96 : 120;
+  cfgfonts.writeEntry( "forceFontDPI", dpi );
+  cfgfonts.writeEntry( "dontChangeAASettings", cbAA->state() == QButton::NoChange );
+  cfgfonts.sync();
+  // if the setting is reset in the module, remove the dpi value,
+  // otherwise don't explicitly remove it and leave any possible system-wide value
+  if( dpi == 0 && dpi_original != 0 ) {
+      KProcIO proc;
+      proc << "xrdb" << "-quiet" << "-remove" << "-nocpp";
+      proc.writeStdin( QCString( "Xft.dpi" ), true );
+      proc.closeWhenDone();
+      proc.start( KProcess::Block );
+  }
+
   // KDE-1.x support
   KSimpleConfig* config = new KSimpleConfig( QDir::homeDirPath() + "/.kderc" );
   config->setGroup( "General" );
@@ -678,12 +728,20 @@ void KFonts::save()
 
   kapp->processEvents(); // Process font change ourselves
 
-  if(aaSettings->save( useAA ) || (useAA != useAA_original) ) {
+  bool aaSave = false;
+  // Don't overwrite global settings unless explicitly asked for - e.g. the system
+  // fontconfig setup may be much more complex than this module can provide.
+  // TODO: With NoChange the changes already made by this module should be reverted somehow.
+  if( cbAA->state() != QButton::NoChange )
+      aaSave = aaSettings->save( useAA == QButton::On );
+
+  if( aaSave || (useAA != useAA_original) || dpi != dpi_original) {
     KMessageBox::information(this,
       i18n(
-        "<p>You have changed anti-aliasing related settings. This change will only affect newly started applications.</p>"
-      ), i18n("Anti-Aliasing Settings Changed"), "AAsettingsChanged", false);
+        "<p>Some changes such as anti-aliasing will only affect newly started applications.</p>"
+      ), i18n("Font Settings Changed"), "FontSettingsChanged", false);
     useAA_original = useAA;
+    dpi_original = dpi;
   }
 
   runRdb(KRdbExportXftSettings);
@@ -708,7 +766,8 @@ void KFonts::slotApplyFontDiff()
 
 void KFonts::slotUseAntiAliasing()
 {
-    useAA = cbAA->isChecked();
+    useAA = cbAA->state();
+    aaSettingsButton->setEnabled( cbAA->state() == QButton::On );
     emit changed(true);
 }
 
