@@ -13,7 +13,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 // Behaviour options for konqueror
@@ -21,18 +21,20 @@
 #include <qcheckbox.h>
 #include <qlabel.h>
 #include <qlayout.h>
-#include <qwhatsthis.h>
+#include <qvbox.h>
 #include <qvbuttongroup.h>
 #include <qvgroupbox.h>
-#include <qvbox.h>
+#include <qwhatsthis.h>
+
+#include <dcopclient.h>
+
+#include <kapplication.h>
 #include <kconfig.h>
+#include <kio/uiserver_stub.h>
 #include <klocale.h>
 #include <konq_defaults.h>
 #include <kstandarddirs.h>
-#include <kapplication.h>
 #include <kurlrequester.h>
-#include <dcopclient.h>
-#include <kio/uiserver_stub.h>
 
 #include "behaviour.h"
 
@@ -41,6 +43,8 @@ KBehaviourOptions::KBehaviourOptions(KConfig *config, QString group, QWidget *pa
 {
     QLabel * label;
 
+    setQuickHelp( i18n("<h1>Konqueror Behavior</h1> You can configure how Konqueror behaves as a file manager here."));
+
     QVBoxLayout *lay = new QVBoxLayout( this, 0, KDialog::spacingHint() );
 
 	QVGroupBox * miscGb = new QVGroupBox(i18n("Misc Options"), this);
@@ -61,7 +65,7 @@ KBehaviourOptions::KBehaviourOptions(KConfig *config, QString group, QWidget *pa
 
     cbNewWin = new QCheckBox(i18n("Open folders in separate &windows"), vbox);
     QWhatsThis::add( cbNewWin, i18n("If this option is checked, Konqueror will open a new window when "
-                                    "you open a folder, rather than showing that folders's contents in the current window."));
+                                    "you open a folder, rather than showing that folder's contents in the current window."));
     connect(cbNewWin, SIGNAL(clicked()), this, SLOT(changed()));
     connect(cbNewWin, SIGNAL(toggled(bool)), SLOT(updateWinPixmap(bool)));
 
@@ -130,6 +134,16 @@ KBehaviourOptions::KBehaviourOptions(KConfig *config, QString group, QWidget *pa
     QWhatsThis::add( label, homestr );
     QWhatsThis::add( homeURL, homestr );
 
+    lay->addItem(new QSpacerItem(0,20,QSizePolicy::Fixed,QSizePolicy::Fixed));
+
+    cbShowDeleteCommand = new QCheckBox( i18n( "Show 'Delete' context me&nu entries which bypass the trashcan" ), this );
+    lay->addWidget( cbShowDeleteCommand );
+    connect(cbShowDeleteCommand, SIGNAL(clicked()), this, SLOT(changed()));
+
+    QWhatsThis::add( cbShowDeleteCommand, i18n("Check this if you want 'Delete' menu commands to be displayed "
+                                                "on the desktop and in the file manager's context menus. "
+						"You can always delete files by holding the Shift key "
+						"while calling 'Move to Trash'."));
 
     QButtonGroup *bg = new QVButtonGroup( i18n("Ask Confirmation For"), this );
     bg->layout()->setSpacing( KDialog::spacingHint() );
@@ -167,6 +181,13 @@ void KBehaviourOptions::slotShowTips(bool b)
 
 void KBehaviourOptions::load()
 {
+	load( false );
+}
+
+void KBehaviourOptions::load( bool useDefaults )
+{
+	 g_pConfig->setReadDefaults( useDefaults );
+
     g_pConfig->setGroup( groupname );
     cbNewWin->setChecked( g_pConfig->readBoolEntry("AlwaysNewWin", false) );
     updateWinPixmap(cbNewWin->isChecked());
@@ -181,6 +202,10 @@ void KBehaviourOptions::load()
     cbShowPreviewsInTips->setChecked( showPreviewsIntips );
 
     cbRenameDirectlyIcon->setChecked( g_pConfig->readBoolEntry("RenameIconDirectly",  DEFAULT_RENAMEICONDIRECTLY ) );
+    
+    KConfig globalconfig("kdeglobals", true, false);
+    globalconfig.setGroup( "KDE" );
+    cbShowDeleteCommand->setChecked( globalconfig.readBoolEntry("ShowDeleteCommand", false) );
 
 //    if (!stips) sbToolTip->setEnabled( false );
     if (!stips) cbShowPreviewsInTips->setEnabled( false );
@@ -195,27 +220,13 @@ void KBehaviourOptions::load()
     g_pConfig->setGroup( "Trash" );
     cbMoveToTrash->setChecked( g_pConfig->readBoolEntry("ConfirmTrash", DEFAULT_CONFIRMTRASH) );
     cbDelete->setChecked( g_pConfig->readBoolEntry("ConfirmDelete", DEFAULT_CONFIRMDELETE) );
+
+	 emit changed( useDefaults );
 }
 
 void KBehaviourOptions::defaults()
 {
-    cbNewWin->setChecked(false);
-
-    homeURL->setURL("~");
-
-    cbListProgress->setChecked( false );
-
-    cbShowTips->setChecked( true );
-    //sbToolTip->setEnabled( true );
-    //sbToolTip->setValue( 6 );
-
-    cbShowPreviewsInTips->setChecked( true );
-    cbShowPreviewsInTips->setEnabled( true );
-
-    cbRenameDirectlyIcon->setChecked( DEFAULT_RENAMEICONDIRECTLY );
-
-    cbMoveToTrash->setChecked( DEFAULT_CONFIRMTRASH );
-    cbDelete->setChecked( DEFAULT_CONFIRMDELETE );
+	load( true );
 }
 
 void KBehaviourOptions::save()
@@ -223,14 +234,19 @@ void KBehaviourOptions::save()
     g_pConfig->setGroup( groupname );
 
     g_pConfig->writeEntry( "AlwaysNewWin", cbNewWin->isChecked() );
-    g_pConfig->writePathEntry( "HomeURL", homeURL->url().isEmpty()? "~" : homeURL->url() );
+    g_pConfig->writePathEntry( "HomeURL", homeURL->url().isEmpty()? QString("~") : homeURL->url() );
 
     g_pConfig->writeEntry( "ShowFileTips", cbShowTips->isChecked() );
     g_pConfig->writeEntry( "ShowPreviewsInFileTips", cbShowPreviewsInTips->isChecked() );
 //    g_pConfig->writeEntry( "FileTipsItems", sbToolTip->value() );
 
     g_pConfig->writeEntry( "RenameIconDirectly", cbRenameDirectlyIcon->isChecked());
-
+    
+    KConfig globalconfig("kdeglobals", false, false);
+    globalconfig.setGroup( "KDE" );
+    globalconfig.writeEntry( "ShowDeleteCommand", cbShowDeleteCommand->isChecked());
+    globalconfig.sync();
+    
     g_pConfig->setGroup( "Trash" );
     g_pConfig->writeEntry( "ConfirmTrash", cbMoveToTrash->isChecked());
     g_pConfig->writeEntry( "ConfirmDelete", cbDelete->isChecked());
@@ -253,6 +269,7 @@ void KBehaviourOptions::save()
     if ( !kapp->dcopClient()->isAttached() )
       kapp->dcopClient()->attach();
     kapp->dcopClient()->send( "konqueror*", "KonquerorIface", "reparseConfiguration()", data );
+    kapp->dcopClient()->send( "kdesktop", "KDesktopIface", "configure()", data );
 }
 
 void KBehaviourOptions::updateWinPixmap(bool b)
@@ -265,15 +282,4 @@ void KBehaviourOptions::updateWinPixmap(bool b)
                                         "kcontrol/pics/onlyone.png")));
 }
 
-QString KBehaviourOptions::quickHelp() const
-{
-    return i18n("<h1>Konqueror Behavior</h1> You can configure how Konqueror behaves as a file manager here.");
-}
-
-
-void KBehaviourOptions::changed()
-{
-  emit KCModule::changed(true);
-}
-
 #include "behaviour.moc"
