@@ -13,8 +13,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #include "printwrapper.h"
@@ -304,9 +304,9 @@ void PrintWrapper::slotPrintRequested(KPrinter *kprinter)
 	for (QStringList::ConstIterator it=files.begin(); it!=files.end(); ++it)
 	{
 		QString	tmpFile;
-		KURL	url(*it);
+		KURL	url = KURL::fromPathOrURL(*it);
 		kdDebug( 500 ) << url.url() << endl;
-		if (KIO::NetAccess::download(url, tmpFile))
+		if (KIO::NetAccess::download(url, tmpFile, this))
 		{
 			filestoprint << tmpFile;
 			kprinter->setDocName(url.fileName());
@@ -319,6 +319,10 @@ void PrintWrapper::slotPrintRequested(KPrinter *kprinter)
 		kprinter->setDocName(force_stdin ? "<STDIN>" : "KPrinter");
 	if (filestoprint.count() == 0)
 	{
+		// At this point force_stdin should be true
+		if (!force_stdin)
+			errormsg(i18n("Nothing to print."));
+
 		// print from stdin
 
 #  if defined(HAVE_SIGSET)
