@@ -83,7 +83,7 @@
 #include "fish.h"
 #include "fishcode.h"
 
-#ifndef NDEBUG 
+#ifndef NDEBUG
 #define myDebug(x) kdDebug(7127) << __LINE__ << ": " x
 #define connected() do{myDebug( << "_______ emitting connected()" << endl); connected();}while(0)
 #define dataReq() do{myDebug( << "_______ emitting dataReq()" << endl); dataReq();}while(0)
@@ -110,6 +110,8 @@ static char *suPath = NULL;
 // disabled: currently not needed. Didn't work reliably.
 // static int isOpenSSH = 0;
 
+static int isNXFish = 0;
+
 #define E(x) ((const char*)remoteEncoding()->encode(x).data())
 
 using namespace KIO;
@@ -133,6 +135,8 @@ int KDE_EXPORT kdemain( int argc, char **argv )
         exit(-1);
     }
 
+    setenv("TZ", "UTC", true);
+
     struct sigaction act;
     memset(&act,0,sizeof(act));
     act.sa_handler = ripper;
@@ -146,6 +150,11 @@ int KDE_EXPORT kdemain( int argc, char **argv )
     ;
     sigaction(SIGCHLD,&act,NULL);
 
+    if (qstrcmp(argv[1],"nxfish")==0) {
+        // Set NXFish - Mode
+        isNXFish=1;
+    }
+
     fishProtocol slave(argv[2], argv[3]);
     slave.dispatchLoop();
 
@@ -244,7 +253,10 @@ fishProtocol::fishProtocol(const QCString &pool_socket, const QCString &app_sock
     if (sshPath == NULL) {
         // disabled: currently not needed. Didn't work reliably.
         // isOpenSSH = !system("ssh -V 2>&1 | grep OpenSSH > /dev/null");
-        sshPath = strdup(QFile::encodeName(KStandardDirs::findExe("ssh")));
+	if (isNXFish)
+            sshPath = strdup(QFile::encodeName(KStandardDirs::findExe("nxfish")));
+	else
+            sshPath = strdup(QFile::encodeName(KStandardDirs::findExe("ssh")));
     }
     if (suPath == NULL) {
         suPath = strdup(QFile::encodeName(KStandardDirs::findExe("su")));
@@ -263,7 +275,6 @@ fishProtocol::fishProtocol(const QCString &pool_socket, const QCString &app_sock
     setMultipleAuthCaching( true );
     connectionAuth.keepPassword = true;
     connectionAuth.url.setProtocol("fish");
-    epoch.setTime_t(0, Qt::UTC);
     outBufPos = -1;
     outBuf = NULL;
     outBufLen = 0;
@@ -272,9 +283,9 @@ fishProtocol::fishProtocol(const QCString &pool_socket, const QCString &app_sock
     mimeAtom.m_uds = UDS_MIME_TYPE;
     mimeAtom.m_long = 0;
     mimeAtom.m_str = QString::null;
-    
+
     hasAppend = false;
-    
+
     isStat = false; // FIXME: just a workaround for konq deficiencies
     redirectUser = ""; // FIXME: just a workaround for konq deficiencies
     redirectPass = ""; // FIXME: just a workaround for konq deficiencies
@@ -297,7 +308,7 @@ Connects to a server and logs us in via SSH. Then starts FISH protocol.
 void fishProtocol::openConnection() {
     if (childPid) return;
 
-    if (connectionHost.isEmpty())
+    if (connectionHost.isEmpty() && !isNXFish)
     {
        error( KIO::ERR_UNKNOWN_HOST, QString::null );
        return;
@@ -473,7 +484,10 @@ bool fishProtocol::connectionStart() {
         FD_SET(childFd,&rfds);
         FD_ZERO(&wfds);
         if (outBufPos >= 0) FD_SET(childFd,&wfds);
-        rc = select(childFd+1, &rfds, &wfds, NULL, NULL);
+        struct timeval timeout;
+        timeout.tv_sec = 0;
+        timeout.tv_usec = 1000;
+        rc = select(childFd+1, &rfds, &wfds, NULL, &timeout);
         if (rc < 0) {
             if (errno == EINTR)
                 continue;
@@ -518,11 +532,13 @@ bool fishProtocol::connectionStart() {
 /**
 writes one chunk of data to stdin of child process
 */
-void fishProtocol::writeChild(const char *buf, int len) {
+void fishProtocol::writeChild(const char *buf, KIO::fileoffset_t len) {
     if (outBufPos >= 0 && outBuf) {
+#if 0
         QString debug;
         debug.setLatin1(outBuf,outBufLen);
         if (len > 0) myDebug( << "write request while old one is pending, throwing away input (" << outBufLen << "," << outBufPos << "," << debug.left(10) << "...)" << endl);
+#endif
         return;
     }
     outBuf = buf;
@@ -533,7 +549,7 @@ void fishProtocol::writeChild(const char *buf, int len) {
 /**
 manages initial communication setup including password queries
 */
-int fishProtocol::establishConnection(char *buffer, int len) {
+int fishProtocol::establishConnection(char *buffer, KIO::fileoffset_t len) {
     QString buf;
     buf.setLatin1(buffer,len);
     int pos;
@@ -585,13 +601,16 @@ int fishProtocol::establishConnection(char *buffer, int len) {
                 myDebug( << "sending mpass" << endl);
                 connectionAuth.prompt = thisFn+buf;
                 if (local)
-                    connectionAuth.caption = i18n("Local Login");
+                    connectionAuth.caption = i18n("Local Login") + " - " + url.user() + "@" + url.host();
                 else
-                    connectionAuth.caption = i18n("SSH Authorization");
-                if ((!firstLogin || !checkCachedAuthentication(connectionAuth)) && !openPassDlg(connectionAuth)) {
-                    error(ERR_USER_CANCELED,connectionHost);
-                    shutdownConnection();
-                    return -1;
+                    connectionAuth.caption = i18n("SSH Authorization") + " - " + url.user() + "@" + url.host();
+                if ((!firstLogin || !checkCachedAuthentication(connectionAuth))) {
+                    connectionAuth.password = QString::null; // don't prefill
+                    if ( !openPassDlg(connectionAuth)) {
+                        error(ERR_USER_CANCELED,connectionHost);
+                        shutdownConnection();
+                        return -1;
+                    }
                 }
                 firstLogin = false;
                 connectionAuth.password += "\n";
@@ -637,7 +656,11 @@ sets connection information for subsequent commands
 void fishProtocol::setHost(const QString & host, int port, const QString & u, const QString & pass){
     QString user(u);
 
-    local = (host == "localhost" && port == 0);
+    if (isNXFish)
+        local = 0;
+    else
+        local = (host == "localhost" && port == 0);
+
     if (port <= 0) port = 0;
     if (user.isEmpty()) user = getenv("LOGNAME");
 
@@ -751,9 +774,7 @@ int fishProtocol::handleResponse(const QString &str){
 
 int fishProtocol::makeTimeFromLs(const QString &monthStr, const QString &dayStr, const QString &timeyearStr)
 {
-    QDateTime dt;
-    dt.setTime_t(time(0));
-    dt.setTime(QTime());
+    QDateTime dt(QDate::currentDate(Qt::UTC));
     int year = dt.date().year();
     int month = dt.date().month();
     int currentMonth = month;
@@ -780,7 +801,7 @@ int fishProtocol::makeTimeFromLs(const QString &monthStr, const QString &dayStr,
     }
     dt.date().setYMD(year,month,day);
 
-    return epoch.secsTo(dt);
+    return dt.toTime_t();
 }
 
 /**
@@ -791,6 +812,7 @@ void fishProtocol::manageConnection(const QString &l) {
     int rc = handleResponse(line);
     UDSAtom atom;
     QDateTime dt;
+    KIO::filesize_t fsize;
     int pos, pos2, pos3;
     bool isOk = false;
     if (!rc) {
@@ -824,10 +846,10 @@ void fishProtocol::manageConnection(const QString &l) {
                 case '7':
                 case '8':
                 case '9':
-                    pos = line.toInt(&isOk);
-                    if (pos > 0 && isOk) errorCount--;
+                    fsize = line.toULongLong(&isOk);
+                    if (fsize > 0 && isOk) errorCount--;
                     if ((fishCommand == FISH_LIST) && (listReason == LIST))
-                        totalSize(pos);
+                        totalSize(fsize);
                     break;
 
                 case 'P':
@@ -912,13 +934,13 @@ void fishProtocol::manageConnection(const QString &l) {
                     if (pos < 0 || pos2 < 0 || pos3 < 0) break;
                     dt.setTime(QTime(line.mid(pos+1,pos2-pos-1).toInt(),line.mid(pos2+1,pos3-pos2-1).toInt(),line.mid(pos3+1).toInt()));
                     errorCount--;
-                    atom.m_long = epoch.secsTo(dt);
+                    atom.m_long = dt.toTime_t();
                     udsEntry.append(atom);
                     break;
 
                 case 'S':
                     atom.m_uds = UDS_SIZE;
-                    atom.m_long = line.mid(1).toLongLong(&isOk);
+                    atom.m_long = line.mid(1).toULongLong(&isOk);
                     if (!isOk) break;
                     errorCount--;
                     udsEntry.append(atom);
@@ -938,25 +960,34 @@ void fishProtocol::manageConnection(const QString &l) {
                     // By default, the mimetype comes from the extension
                     // We'll use the file(1) result only as fallback [like the rest of KDE does]
                     {
-                      KMimeType::Ptr mime = KMimeType::findByURL( KURL("fish://host/" + thisFn) );
+                      KURL kurl("fish://host/");
+                      kurl.setFileName(thisFn);  // properly encode special chars
+                      KMimeType::Ptr mime = KMimeType::findByURL(kurl);
                       if ( mime->name() != KMimeType::defaultMimeType() )
                           mimeAtom.m_str = mime->name();
                     }
                     errorCount--;
                     break;
 
-                case 'M':
+                case 'M': {
+                    QString type = line.mid(1);
+
+                    // First thing's first. If remote says this is a directory, throw out any
+                    // name-based file type guesses.
+                    if (type == "inode/directory" && mimeAtom.m_str != type) {
+                        mimeAtom.m_str = type;
+                        typeAtom.m_long = S_IFDIR;
+                    }
                     // This is getting ugly. file(1) makes some uneducated
                     // guesses, so we must try to ignore them (#51274)
-                    if (mimeAtom.m_str.isEmpty() && line.right(8) != "/unknown" &&
+                    else if (mimeAtom.m_str.isEmpty() && line.right(8) != "/unknown" &&
                             (thisFn.find('.') < 0 || (line.left(8) != "Mtext/x-"
                                                   && line != "Mtext/plain"))) {
-                        mimeAtom.m_str = line.mid(1);
-                        if ( mimeAtom.m_str == "inode/directory" ) // a symlink to a dir is a dir
-                          typeAtom.m_long = S_IFDIR;
+                        mimeAtom.m_str = type;
                     }
                     errorCount--;
                     break;
+                }
 
                 case 'L':
                     atom.m_uds = UDS_LINK_DEST;
@@ -991,7 +1022,7 @@ void fishProtocol::manageConnection(const QString &l) {
                 recvLen = 0;
                 break;
             }
-            recvLen = line.toInt(&isOk);
+            recvLen = line.toLongLong(&isOk);
             if (!isOk) {
                 error(ERR_COULD_NOT_READ,url.prettyURL());
                 shutdownConnection();
@@ -1016,7 +1047,6 @@ void fishProtocol::manageConnection(const QString &l) {
                 shutdownConnection();
             } else {
                 rawRead = recvLen;
-                t_start = t_last = time(NULL);
                 dataRead = 0;
                 mimeTypeSent = false;
                 if (recvLen == 0)
@@ -1167,7 +1197,7 @@ void fishProtocol::sent()
 {
     if (rawWrite > 0) {
         myDebug( << "writing raw: " << rawData.size() << "/" << rawWrite << endl);
-        writeChild(rawData.data(),((unsigned int)rawWrite > rawData.size()?rawData.size():rawWrite));
+        writeChild(rawData.data(),(rawWrite > rawData.size()?rawData.size():rawWrite));
         rawWrite -= rawData.size();
         if (rawWrite > 0) {
             dataReq();
@@ -1195,14 +1225,14 @@ void fishProtocol::sent()
     }
 }
 
-int fishProtocol::received(const char *buffer, int buflen)
+int fishProtocol::received(const char *buffer, KIO::fileoffset_t buflen)
 {
     int pos = 0;
     do {
         if (buflen <= 0) break;
 
         if (rawRead > 0) {
-            myDebug( << "processedSize " << dataRead << ", len " << buflen << "/" << rawRead << endl);
+            //myDebug( << "processedSize " << dataRead << ", len " << buflen << "/" << rawRead << endl);
             int dataSize = (rawRead > buflen?buflen:rawRead);
             if (!mimeTypeSent)
             {
@@ -1219,7 +1249,23 @@ int fishProtocol::received(const char *buffer, int buflen)
                     myDebug( << "wait for more" << endl);
                     break;
                 }
-                sendmimeType(KMimeMagic::self()->findBufferFileType(mimeBuffer,url.path())->mimeType());
+
+                // We need a KMimeType::findByNameAndContent(filename,data)
+                // For now we do: find by extension, and if not found (or extension not reliable)
+                // then find by content.
+                bool accurate = false;
+                KMimeType::Ptr mime = KMimeType::findByURL( url, 0, false, true, &accurate );
+                if ( !mime || mime->name() == KMimeType::defaultMimeType()
+                     || !accurate )
+                {
+                    KMimeType::Ptr p_mimeType = KMimeType::findByContent(mimeBuffer);
+                    if ( p_mimeType && p_mimeType->name() != KMimeType::defaultMimeType() )
+                        mime = p_mimeType;
+                }
+
+                sendmimeType(mime->name());
+
+
                 mimeTypeSent = true;
                 if (fishCommand != FISH_READ) {
                     totalSize(dataRead + rawRead);
@@ -1234,15 +1280,10 @@ int fishProtocol::received(const char *buffer, int buflen)
             QByteArray bdata;
             bdata.duplicate(buffer,dataSize);
             data(bdata);
-            
+
             dataRead += dataSize;
             rawRead -= dataSize;
-            time_t t = time(NULL);
-            if (t-t_last >= 1) {
-                processedSize(dataRead);
-                //speed(dataRead/(t-t_start));
-                t_last = t;
-            }
+            processedSize(dataRead);
             if (rawRead <= 0) {
                 buffer += dataSize;
                 buflen -= dataSize;
@@ -1257,16 +1298,16 @@ int fishProtocol::received(const char *buffer, int buflen)
         // Find newline
         while((pos < buflen) && (buffer[pos] != '\n'))
             ++pos;
-        
+
         if (pos < buflen)
         {
            QString s = remoteEncoding()->decode(QCString(buffer,pos+1));
-        
+
            buffer += pos+1;
            buflen -= pos+1;
 
            manageConnection(s);
-           
+
            pos = 0;
            // Find next newline
            while((pos < buflen) && (buffer[pos] != '\n'))
@@ -1354,7 +1395,10 @@ void fishProtocol::run() {
             FD_SET(childFd,&rfds);
             FD_ZERO(&wfds);
             if (outBufPos >= 0) FD_SET(childFd,&wfds);
-            rc = select(childFd+1, &rfds, &wfds, NULL, NULL);
+            struct timeval timeout;
+            timeout.tv_sec = 0;
+            timeout.tv_usec = 1000;
+            rc = select(childFd+1, &rfds, &wfds, NULL, &timeout);
             if (rc < 0) {
                 if (errno == EINTR)
                     continue;
@@ -1364,9 +1408,11 @@ void fishProtocol::run() {
                 return;
             }
             if (FD_ISSET(childFd,&wfds) && outBufPos >= 0) {
+#if 0
                 QString debug;
                 debug.setLatin1(outBuf+outBufPos,outBufLen-outBufPos);
                 myDebug( << "now writing " << (outBufLen-outBufPos) << " " << debug.left(40) << "..." << endl);
+#endif
                 if (outBufLen-outBufPos > 0) rc = write(childFd,outBuf+outBufPos,outBufLen-outBufPos);
                 else rc = 0;
                 if (rc >= 0) outBufPos += rc;
