@@ -90,7 +90,7 @@ void WinInfo::changeState( unsigned long state, unsigned long mask )
 // ****************************************
 
 RootInfo::RootInfo( Workspace* ws, Display *dpy, Window w, const char *name, unsigned long pr[], int pr_num, int scr )
-    : NETRootInfo3( dpy, w, name, pr, pr_num, scr )
+    : NETRootInfo4( dpy, w, name, pr, pr_num, scr )
     {
     workspace = ws;
     }
@@ -180,6 +180,11 @@ void RootInfo::gotPing( Window w, Time timestamp )
         c->gotPing( timestamp );
     }
 
+void RootInfo::changeShowingDesktop( bool showing )
+    {
+    workspace->setShowingDesktop( showing );
+    }
+
 // ****************************************
 // Workspace
 // ****************************************
@@ -362,33 +367,26 @@ bool Workspace::workspaceEvent( XEvent * e )
                     XMapRaised( qt_xdisplay(), e->xmaprequest.window );
                 return true;
                 }
-            if ( c ) 
+            if( c )
                 {
                 c->windowEvent( e );
-                if ( !c->wantsTabFocus())
-                    focus_chain.remove( c );  // TODO move focus_chain changes to functions
+                updateFocusChains( c, FocusChainUpdate );
                 return true;
                 }
             break;
             }
         case EnterNotify:
-            if ( QWhatsThis::inWhatsThisMode() )
             {
-            QWidget* w = QWidget::find( e->xcrossing.window );
-            if ( w )
-                QWhatsThis::leaveWhatsThisMode();
-            }
-
-            if (electric_have_borders &&
-               (e->xcrossing.window == electric_top_border ||
-                e->xcrossing.window == electric_left_border ||
-                e->xcrossing.window == electric_bottom_border ||
-                e->xcrossing.window == electric_right_border))
+            if ( QWhatsThis::inWhatsThisMode() )
                 {
-                // the user entered an electric border
-                electricBorder(e);
+                QWidget* w = QWidget::find( e->xcrossing.window );
+                if ( w )
+                    QWhatsThis::leaveWhatsThisMode();
                 }
+            if( electricBorder(e))
+                return true;
             break;
+            }
         case LeaveNotify:
             {
             if ( !QWhatsThis::inWhatsThisMode() )
@@ -404,15 +402,15 @@ bool Workspace::workspaceEvent( XEvent * e )
             if ( e->xconfigurerequest.parent == root ) 
                 {
                 XWindowChanges wc;
-                unsigned int value_mask = 0;
-                wc.border_width = 0;
+                wc.border_width = e->xconfigurerequest.border_width;
                 wc.x = e->xconfigurerequest.x;
                 wc.y = e->xconfigurerequest.y;
                 wc.width = e->xconfigurerequest.width;
                 wc.height = e->xconfigurerequest.height;
                 wc.sibling = None;
                 wc.stack_mode = Above;
-                value_mask = e->xconfigurerequest.value_mask | CWBorderWidth;
+                unsigned int value_mask = e->xconfigurerequest.value_mask
+                    & ( CWX | CWY | CWWidth | CWHeight | CWBorderWidth );
                 XConfigureWindow( qt_xdisplay(), e->xconfigurerequest.window, value_mask, &wc );
                 return true;
                 }
@@ -437,8 +435,11 @@ bool Workspace::workspaceEvent( XEvent * e )
                 if( focus == None || focus == PointerRoot )
                     {
                     //kdWarning( 1212 ) << "X focus set to None/PointerRoot, reseting focus" << endl;
-                    if( mostRecentlyActivatedClient() != NULL )
-                        requestFocus( mostRecentlyActivatedClient(), true );
+                    Client *c = mostRecentlyActivatedClient();
+                    if( c != NULL )
+                        requestFocus( c, true );
+                    else if( activateNextClient( NULL ))
+                        ; // ok, activated
                     else
                         focusToNull();
                     }
@@ -446,6 +447,14 @@ bool Workspace::workspaceEvent( XEvent * e )
             // fall through
         case FocusOut:
             return true; // always eat these, they would tell Qt that KWin is the active app
+        case ClientMessage:
+            if( electricBorder( e ))
+                return true;
+            break;
+        case MappingNotify:
+            XRefreshKeyboardMapping( &e->xmapping );
+            tab_box->updateKeyMapping();
+            break;
         default:
             break;
         }
@@ -537,6 +546,11 @@ bool Client::windowEvent( XEvent* e )
             }
         if(( dirty[ WinInfo::PROTOCOLS2 ] & NET::WM2StartupId ) != 0 )
             startupIdChanged();
+        if( dirty[ WinInfo::PROTOCOLS ] & NET::WMIconGeometry )
+            {
+            if( demandAttentionKNotifyTimer != NULL )
+                demandAttentionKNotify();
+            }
         }
 
 // TODO move all focus handling stuff to separate file?
@@ -566,7 +580,7 @@ bool Client::windowEvent( XEvent* e )
             workspace()->setWasUserInteraction();
             buttonPressEvent( e->xbutton.window, e->xbutton.button, e->xbutton.state,
                 e->xbutton.x, e->xbutton.y, e->xbutton.x_root, e->xbutton.y_root );
-            break;;
+            break;
         case KeyRelease:
     // don't update user time on releases
     // e.g. if the user presses Alt+F2, the Alt release
@@ -617,9 +631,6 @@ bool Client::windowEvent( XEvent* e )
                 workspace()->updateColormap();
             }
             break;
-        case VisibilityNotify:
-            visibilityNotifyEvent( &e->xvisibility );
-            break;
         default:
             if( e->xany.window == window())
             {
@@ -938,8 +949,7 @@ void Client::leaveNotifyEvent( XCrossingEvent* e )
             {
             cancelAutoRaise();
             workspace()->cancelDelayFocus();
-            delete shadeHoverTimer;
-            shadeHoverTimer = 0;
+            cancelShadeHover();
             if ( shade_mode == ShadeHover && !moveResizeMode && !buttonDown )
                setShade( ShadeNormal );
             }
@@ -995,11 +1005,14 @@ void Client::ungrabButton( int modifier )
   (Motif, AWT, Tk, ...)
  */
 void Client::updateMouseGrab()
-    {                   // see Workspace::establishTabBoxGrab()
-    if( isActive() && !workspace()->forcedGlobalMouseGrab())
+    {
+    if( isActive() && !workspace()->forcedGlobalMouseGrab()) // see Workspace::establishTabBoxGrab()
         {
         // remove the grab for no modifiers only if the window
         // is unobscured or if the user doesn't want click raise
+        // (it is unobscured if it the topmost in the unconstrained stacking order, i.e. it is
+        // the most recently raised window)
+        bool not_obscured = workspace()->topClientOnDesktop( workspace()->currentDesktop(), true, false ) == this;
         if( !options->clickRaise || not_obscured )
             ungrabButton( None );
         else
@@ -1075,6 +1088,15 @@ bool Client::eventFilter( QObject* o, QEvent* e )
         return motionNotifyEvent( decorationId(), qtToX11State( ev->state()),
             ev->x(), ev->y(), ev->globalX(), ev->globalY() );
         }
+    if( e->type() == QEvent::Wheel )
+        {
+        QWheelEvent* ev = static_cast< QWheelEvent* >( e );
+        bool r = buttonPressEvent( decorationId(), ev->delta() > 0 ? Button4 : Button5, qtToX11State( ev->state()),
+            ev->x(), ev->y(), ev->globalX(), ev->globalY() );
+        r = r || buttonReleaseEvent( decorationId(), ev->delta() > 0 ? Button4 : Button5, qtToX11State( ev->state()),
+            ev->x(), ev->y(), ev->globalX(), ev->globalY() );
+        return r;
+        }
     if( e->type() == QEvent::Resize )
         {
         QResizeEvent* ev = static_cast< QResizeEvent* >( e );
@@ -1133,6 +1155,10 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
                 case Button3:
                     com = options->commandAll3();
                     break;
+                case Button4:
+                case Button5:
+                    com = options->operationWindowMouseWheel( button == Button4 ? 120 : -120 );
+                    break;
                 }
             }
         else
@@ -1169,11 +1195,11 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
             {
             bool replay = performMouseCommand( com, QPoint( x_root, y_root), perform_handled );
 
-            if ( isSpecialWindow() && !isOverride())
+            if ( isSpecialWindow())
                 replay = TRUE;
 
-                if( w == wrapperId()) // these can come only from a grab
-                    XAllowEvents(qt_xdisplay(), replay? ReplayPointer : SyncPointer, CurrentTime ); //qt_x_time);
+            if( w == wrapperId()) // these can come only from a grab
+                XAllowEvents(qt_xdisplay(), replay? ReplayPointer : SyncPointer, CurrentTime ); //qt_x_time);
             return true;
             }
         }
@@ -1206,10 +1232,11 @@ void Client::processDecorationButtonPress( int button, int /*state*/, int x, int
         com = active ? options->commandActiveTitlebar2() : options->commandInactiveTitlebar2();
     else if ( button == Button3 )
         com = active ? options->commandActiveTitlebar3() : options->commandInactiveTitlebar3();
-    if( com != Options::MouseOperationsMenu // actions where it's not possible to get the matching
+    if( button == Button1
+        && com != Options::MouseOperationsMenu // actions where it's not possible to get the matching
         && com != Options::MouseMinimize )  // mouse release event
         {
-// FRAME      mouseMoveEvent( e ); shouldn't be necessary
+        mode = mousePosition( QPoint( x, y ));
         buttonDown = TRUE;
         moveOffset = QPoint( x, y );
         invertedMoveOffset = rect().bottomRight() - moveOffset;
@@ -1319,6 +1346,9 @@ bool Client::motionNotifyEvent( Window w, int /*state*/, int x, int y, int x_roo
         if( newmode != mode )
             setCursor( newmode );
         mode = newmode;
+        // reset the timestamp for the optimization, otherwise with long passivity
+        // the option in waitingMotionEvent() may be always true
+        next_motion_time = CurrentTime;
         return false;
         }
     if( w == moveResizeGrabWindow())
@@ -1417,22 +1447,17 @@ void Client::focusOutEvent( XFocusOutEvent* e )
         setActive( FALSE );
     }
 
-void Client::visibilityNotifyEvent( XVisibilityEvent * e)
-    {
-    if( e->window != frameId())
-        return; // care only about the whole frame
-    bool new_not_obscured = e->state == VisibilityUnobscured;
-    if( not_obscured == new_not_obscured )
-        return;
-    not_obscured = new_not_obscured;
-    updateMouseGrab();
-    }
-
 // performs _NET_WM_MOVERESIZE
 void Client::NETMoveResize( int x_root, int y_root, NET::Direction direction )
     {
     if( direction == NET::Move )
         performMouseCommand( Options::MouseMove, QPoint( x_root, y_root ));
+    else if( moveResizeMode && direction == NET::MoveResizeCancel )
+    {
+        finishMoveResize( true );
+        buttonDown = FALSE;
+        setCursor( mode );
+    }
     else if( direction >= NET::TopLeft && direction <= NET::Left ) 
         {
         static const Position convert[] =
