@@ -55,6 +55,7 @@ extern int screen_number;
 Workspace *Workspace::_self = 0;
 
 KProcess* kompmgr = 0;
+KSelectionOwner* kompmgr_selection;
 
 bool allowKompmgrRestart = TRUE;
 
@@ -62,7 +63,7 @@ bool allowKompmgrRestart = TRUE;
 // It's a nightmare to understand, especially with so few comments :(
 
 // Matthias: Feel free to ask me questions about it. Feel free to add
-// comments. I dissagree that further splittings makes it easier. 2500
+// comments. I disagree that further splittings makes it easier. 2500
 // lines are not too much. It's the task that is complex, not the
 // code.
 Workspace::Workspace( bool restore )
@@ -437,7 +438,11 @@ Workspace::~Workspace()
         {
 	// only release the window
         (*it)->releaseWindow( true );
-        // no removeClient() is called !
+        // No removeClient() is called, it does more than just removing.
+        // However, remove from some lists to e.g. prevent performTransiencyCheck()
+        // from crashing.
+        clients.remove( *it );
+        desktops.remove( *it );
         }
     delete desktop_widget;
     delete tab_box;
@@ -512,8 +517,7 @@ void Workspace::addClient( Client* c, allowed_t )
         }
     else
         {
-        if ( c->wantsTabFocus() && !focus_chain.contains( c ))
-            focus_chain.append( c );
+        updateFocusChains( c, FocusChainUpdate ); // add to focus chain if not already there
         clients.append( c );
         }
     if( !unconstrained_stacking_order.contains( c ))
@@ -536,6 +540,7 @@ void Workspace::addClient( Client* c, allowed_t )
     updateStackingOrder( true ); // propagate new client
     if( c->isUtility() || c->isMenu() || c->isToolbar())
         updateToolWindows( true );
+    checkNonExistentClients();
     }
 
 /*
@@ -561,7 +566,11 @@ void Workspace::removeClient( Client* c, allowed_t )
     desktops.remove( c );
     unconstrained_stacking_order.remove( c );
     stacking_order.remove( c );
-    focus_chain.remove( c );
+    for( int i = 1;
+         i <= numberOfDesktops();
+         ++i )
+        focus_chain[ i ].remove( c );
+    global_focus_chain.remove( c );
     attention_chain.remove( c );
     if( c->isTopMenu())
         removeTopMenu( c );
@@ -588,6 +597,89 @@ void Workspace::removeClient( Client* c, allowed_t )
     updateClientArea();
     }
 
+void Workspace::updateFocusChains( Client* c, FocusChainChange change )
+    {
+    if( !c->wantsTabFocus()) // doesn't want tab focus, remove
+        {
+        for( int i=1;
+             i<= numberOfDesktops();
+             ++i )
+            focus_chain[i].remove(c);
+        global_focus_chain.remove( c );
+        return;
+        }
+    if(c->desktop() == NET::OnAllDesktops)
+        { //now on all desktops, add it to focus_chains it is not already in
+        for( int i=1; i<= numberOfDesktops(); i++)
+            { // making first/last works only on current desktop, don't affect all desktops
+            if( i == currentDesktop()
+                && ( change == FocusChainMakeFirst || change == FocusChainMakeLast ))
+                {
+                focus_chain[ i ].remove( c );
+                if( change == FocusChainMakeFirst )
+                    focus_chain[ i ].append( c );
+                else
+                    focus_chain[ i ].prepend( c );
+                }
+            else if( !focus_chain[ i ].contains( c ))
+                { // add it after the active one
+                if( active_client != NULL && active_client != c
+                    && !focus_chain[ i ].isEmpty() && focus_chain[ i ].last() == active_client )
+                    focus_chain[ i ].insert( focus_chain[ i ].fromLast(), c );
+                else
+                    focus_chain[ i ].append( c ); // otherwise add as the first one
+                }
+            }
+        }
+    else    //now only on desktop, remove it anywhere else
+        {
+        for( int i=1; i<= numberOfDesktops(); i++)
+            {
+            if( i == c->desktop())
+                {
+                if( change == FocusChainMakeFirst )
+                    {
+                    focus_chain[ i ].remove( c );
+                    focus_chain[ i ].append( c );
+                    }
+                else if( change == FocusChainMakeLast )
+                    {
+                    focus_chain[ i ].remove( c );
+                    focus_chain[ i ].prepend( c );
+                    }
+                else if( !focus_chain[ i ].contains( c ))
+                    {
+                    if( active_client != NULL && active_client != c
+                        && !focus_chain[ i ].isEmpty() && focus_chain[ i ].last() == active_client )
+                        focus_chain[ i ].insert( focus_chain[ i ].fromLast(), c );
+                    else
+                        focus_chain[ i ].append( c ); // otherwise add as the first one
+                    }
+                }
+            else
+                focus_chain[ i ].remove( c );
+            }
+        }
+    if( change == FocusChainMakeFirst )
+        {
+        global_focus_chain.remove( c );
+        global_focus_chain.append( c );
+        }
+    else if( change == FocusChainMakeLast )
+        {
+        global_focus_chain.remove( c );
+        global_focus_chain.prepend( c );
+        }
+    else if( !global_focus_chain.contains( c ))
+        {
+        if( active_client != NULL && active_client != c
+            && !global_focus_chain.isEmpty() && global_focus_chain.last() == active_client )
+            global_focus_chain.insert( global_focus_chain.fromLast(), c );
+        else
+            global_focus_chain.append( c ); // otherwise add as the first one
+        }
+    }
+
 void Workspace::updateCurrentTopMenu()
     {
     if( !managingTopMenus())
@@ -914,6 +1006,8 @@ void Workspace::loadDesktopSettings()
     screenarea = NULL;
     rootInfo->setNumberOfDesktops( number_of_desktops );
     desktop_focus_chain.resize( n );
+    // make it +1, so that it can be accessed as [1..numberofdesktops]
+    focus_chain.resize( n + 1 );
     for(int i = 1; i <= n; i++) 
         {
         QString s = c->readEntry(QString("Name_%1").arg(i),
@@ -1138,30 +1232,19 @@ bool Workspace::setCurrentDesktop( int new_desktop )
     if ( options->focusPolicyIsReasonable()) 
         {
         // Search in focus chain
-
-        if ( focus_chain.contains( active_client ) && active_client->isShown( true )
-            && active_client->isOnCurrentDesktop())
+        if ( movingClient != NULL && active_client == movingClient
+            && focus_chain[currentDesktop()].contains( active_client )
+            && active_client->isShown( true ) && active_client->isOnCurrentDesktop())
             {
             c = active_client; // the requestFocus below will fail, as the client is already active
             }
-
         if ( !c ) 
             {
-            for( ClientList::ConstIterator it = focus_chain.fromLast(); it != focus_chain.end(); --it) 
+            for( ClientList::ConstIterator it = focus_chain[currentDesktop()].fromLast();
+                 it != focus_chain[currentDesktop()].end();
+                 --it )
                 {
-                if ( (*it)->isShown( false ) && !(*it)->isOnAllDesktops() && (*it)->isOnCurrentDesktop()) 
-                    {
-                    c = *it;
-                    break;
-                    }
-                }
-            }
-
-        if ( !c ) 
-            {
-            for( ClientList::ConstIterator it = focus_chain.fromLast(); it != focus_chain.end(); --it) 
-                {
-                if ( (*it)->isShown( false ) && (*it)->isOnCurrentDesktop()) 
+                if ( (*it)->isShown( false ) && (*it)->isOnCurrentDesktop())
                     {
                     c = *it;
                     break;
@@ -1181,28 +1264,21 @@ bool Workspace::setCurrentDesktop( int new_desktop )
 
     if ( c ) 
         requestFocus( c );
+    else if( !desktops.isEmpty() ) 
+        requestFocus( findDesktop( true, currentDesktop()));
     else 
         focusToNull();
 
-    if( !desktops.isEmpty() ) 
-        {
-        Window w_tmp;
-        int i_tmp;
-        XGetInputFocus( qt_xdisplay(), &w_tmp, &i_tmp );
-        if( w_tmp == null_focus_window ) // CHECKME?
-            requestFocus( findDesktop( true, currentDesktop()));
-        }
-
     updateCurrentTopMenu();
 
     // Update focus chain:
     //  If input: chain = { 1, 2, 3, 4 } and current_desktop = 3,
     //   Output: chain = { 3, 1, 2, 4 }.
-//    kdDebug(1212) << QString("Switching to desktop #%1, at focus_chain index %2\n")
-//      .arg(current_desktop).arg(desktop_focus_chain.find( current_desktop ));
-    for( int i = desktop_focus_chain.find( current_desktop ); i > 0; i-- )
+//    kdDebug(1212) << QString("Switching to desktop #%1, at focus_chain[currentDesktop()] index %2\n")
+//      .arg(currentDesktop()).arg(desktop_focus_chain.find( currentDesktop() ));
+    for( int i = desktop_focus_chain.find( currentDesktop() ); i > 0; i-- )
         desktop_focus_chain[i] = desktop_focus_chain[i-1];
-    desktop_focus_chain[0] = current_desktop;
+    desktop_focus_chain[0] = currentDesktop();
 
 //    QString s = "desktop_focus_chain[] = { ";
 //    for( uint i = 0; i < desktop_focus_chain.size(); i++ )
@@ -1375,6 +1451,7 @@ void Workspace::setNumberOfDesktops( int n )
         rootInfo->setDesktopViewport( number_of_desktops, *viewports );
         delete[] viewports;
         updateClientArea( true );
+        focus_chain.resize( number_of_desktops + 1 );
         }
 
     // if the number of desktops decreased, move all
@@ -1396,6 +1473,7 @@ void Workspace::setNumberOfDesktops( int n )
         rootInfo->setDesktopViewport( number_of_desktops, *viewports );
         delete[] viewports;
         updateClientArea( true );
+        focus_chain.resize( number_of_desktops + 1 );
         }
 
     saveDesktopSettings();
@@ -1431,9 +1509,6 @@ void Workspace::sendClientToDesktop( Client* c, int desk, bool dont_activate )
     else 
         {
         raiseClient( c );
-        focus_chain.remove( c );
-        if ( c->wantsTabFocus() )
-            focus_chain.append( c );
         }
 
     ClientList transients_stacking_order = ensureStackingOrder( c->transients());
@@ -1716,35 +1791,35 @@ unsigned int Workspace::sendFakedMouseEvent( QPoint pos, WId w, MouseEmulation t
         XTranslateCoordinates( qt_xdisplay(), qt_xrootwin(), w, pos.x(), pos.y(), &x, &y, &xw );
         if ( type == EmuMove ) 
             { // motion notify events
-            XMotionEvent e;
+            XEvent e;
             e.type = MotionNotify;
-            e.window = w;
-            e.root = qt_xrootwin();
-            e.subwindow = w;
-            e.time = qt_x_time;
-            e.x = x;
-            e.y = y;
-            e.x_root = pos.x();
-            e.y_root = pos.y();
-            e.state = state;
-            e.is_hint = NotifyNormal;
-            XSendEvent( qt_xdisplay(), w, TRUE, ButtonMotionMask, (XEvent*)&e );
+            e.xmotion.window = w;
+            e.xmotion.root = qt_xrootwin();
+            e.xmotion.subwindow = w;
+            e.xmotion.time = qt_x_time;
+            e.xmotion.x = x;
+            e.xmotion.y = y;
+            e.xmotion.x_root = pos.x();
+            e.xmotion.y_root = pos.y();
+            e.xmotion.state = state;
+            e.xmotion.is_hint = NotifyNormal;
+            XSendEvent( qt_xdisplay(), w, TRUE, ButtonMotionMask, &e );
             }
         else 
             {
-            XButtonEvent e;
+            XEvent e;
             e.type = type == EmuRelease ? ButtonRelease : ButtonPress;
-            e.window = w;
-            e.root = qt_xrootwin();
-            e.subwindow = w;
-            e.time = qt_x_time;
-            e.x = x;
-            e.y = y;
-            e.x_root = pos.x();
-            e.y_root = pos.y();
-            e.state = state;
-            e.button = button;
-            XSendEvent( qt_xdisplay(), w, TRUE, ButtonPressMask, (XEvent*)&e );
+            e.xbutton.window = w;
+            e.xbutton.root = qt_xrootwin();
+            e.xbutton.subwindow = w;
+            e.xbutton.time = qt_x_time;
+            e.xbutton.x = x;
+            e.xbutton.y = y;
+            e.xbutton.x_root = pos.x();
+            e.xbutton.y_root = pos.y();
+            e.xbutton.state = state;
+            e.xbutton.button = button;
+            XSendEvent( qt_xdisplay(), w, TRUE, ButtonPressMask, &e );
 
             if ( type == EmuPress ) 
                 {
@@ -1946,8 +2021,7 @@ void Workspace::createBorderWindows()
     XSetWindowAttributes attributes;
     unsigned long valuemask;
     attributes.override_redirect = True;
-    attributes.event_mask =  (EnterWindowMask | LeaveWindowMask |
-                              VisibilityChangeMask);
+    attributes.event_mask =  ( EnterWindowMask | LeaveWindowMask );
     valuemask=  (CWOverrideRedirect | CWEventMask | CWCursor );
     attributes.cursor = XCreateFontCursor(qt_xdisplay(),
                                           XC_sb_up_arrow);
@@ -2362,6 +2436,12 @@ void Workspace::startKompmgr()
         }
     else
         {
+        delete kompmgr_selection;
+        char selection_name[ 100 ];
+        sprintf( selection_name, "_NET_WM_CM_S%d", DefaultScreen( qt_xdisplay()));
+        kompmgr_selection = new KSelectionOwner( selection_name );
+        connect( kompmgr_selection, SIGNAL( lostOwnership()), SLOT( stopKompmgr()));
+        kompmgr_selection->claim( true );
         connect(kompmgr, SIGNAL(processExited(KProcess*)), SLOT(restartKompmgr()));
         options->useTranslucency = TRUE;
         allowKompmgrRestart = FALSE;
@@ -2378,6 +2458,8 @@ void Workspace::stopKompmgr()
 {
     if (!kompmgr  || !kompmgr->isRunning())
         return;
+    delete kompmgr_selection;
+    kompmgr_selection = NULL;
     kompmgr->disconnect(this, SLOT(restartKompmgr()));
     options->useTranslucency = FALSE;
     if (popup){ delete popup; popup = 0L; } // to add/remove opacity slider
@@ -2403,6 +2485,8 @@ void Workspace::restartKompmgr()
 {
     if (!allowKompmgrRestart) // uh-ohh
         {
+        delete kompmgr_selection;
+        kompmgr_selection = NULL;
         options->useTranslucency = FALSE;
         KProcess proc;
         proc << "kdialog" << "--error"
@@ -2423,6 +2507,8 @@ void Workspace::restartKompmgr()
 // -------------------
     if (!kompmgr->start(KProcess::NotifyOnExit, KProcess::Stderr))
         {
+        delete kompmgr_selection;
+        kompmgr_selection = NULL;
         options->useTranslucency = FALSE;
         KProcess proc;
         proc << "kdialog" << "--error"
@@ -2549,7 +2635,10 @@ void Workspace::setShowingDesktop( bool showing )
 // Unminimizing a window resets the state but keeps the windows hidden (except
 // the one that was unminimized).
 // A new window resets the state and shows the windows again, with the new window
-// being active.
+// being active. Due to popular demand (#67406) by people who apparently
+// don't see a difference between "show desktop" and "minimize all", this is not
+// true if "showDesktopIsMinimizeAll" is set in kwinrc. In such case showing
+// a new window resets the state but doesn't show windows.
 void Workspace::resetShowingDesktop( bool keep_hidden )
     {
     if( block_showing_desktop > 0 )
