@@ -181,7 +181,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
 
         ClientList ensureStackingOrder( const ClientList& clients ) const;
 
-        Client* topClientOnDesktop( int desktop, bool unconstrained = false ) const;
+        Client* topClientOnDesktop( int desktop, bool unconstrained = false, bool only_normal = true ) const;
         Client* findDesktop( bool topmost, int desktop ) const;
         void sendClientToDesktop( Client* c, int desktop, bool dont_activate );
         void windowToPreviousDesktop( Client* c );
@@ -234,7 +234,6 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void sendPingToWindow( Window w, Time timestamp ); // called from Client::pingWindow()
         void sendTakeActivity( Client* c, Time timestamp, long flags ); // called from Client::takeActivity()
         
-        void stopKompmgr();
         bool kompmgrIsRunning();
         void setOpacity(unsigned long winId, unsigned int opacityPercent);
         void setShadowSize(unsigned long winId, unsigned int shadowSizePercent);
@@ -382,6 +381,7 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         void unblockKompmgrRestart();
         void restartKompmgr();
         void handleKompmgrOutput( KProcess *proc, char *buffer, int buflen);
+        void stopKompmgr();
         // end 
 
     protected:
@@ -508,9 +508,9 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
         ClientList clients;
         ClientList desktops;
 
-        ClientList unconstrained_stacking_order;
-        ClientList stacking_order;
-        QValueVector< ClientList > focus_chain;
+        ClientList unconstrained_stacking_order; // topmost last
+        ClientList stacking_order; // topmost last
+        QValueVector< ClientList > focus_chain; // currently active last
         ClientList global_focus_chain; // this one is only for things like tabbox's MRU
         ClientList should_get_focus; // last is most recent
         ClientList attention_chain;
@@ -629,6 +629,9 @@ class Workspace : public QObject, public KWinInterface, public KDecorationDefine
      signals:
         void kompmgrStarted();
         void kompmgrStopped();
+
+    private:
+        friend bool performTransiencyCheck();
     };
 
 // helper for Workspace::blockStackingUpdates() being called in pairs (true/false)
