@@ -481,5 +481,36 @@ void set_repeatrate(int delay, double rate)
 }
 #endif
 
+void KeyboardConfig::init_keyboard()
+{
+	KConfig *config = new KConfig("kcminputrc", true); // Read-only, no globals
+	config->setGroup("Keyboard");
+
+	XKeyboardState   kbd;
+	XKeyboardControl kbdc;
+
+	XGetKeyboardControl(kapp->getDisplay(), &kbd);
+	bool key = config->readBoolEntry("KeyboardRepeating", true);
+	kbdc.key_click_percent = config->readNumEntry("ClickVolume", kbd.key_click_percent);
+	kbdc.auto_repeat_mode = (key ? AutoRepeatModeOn : AutoRepeatModeOff);
+
+	XChangeKeyboardControl(kapp->getDisplay(),
+						   KBKeyClickPercent | KBAutoRepeatMode,
+						   &kbdc);
+
+	if( key ) {
+		int delay_ = config->readNumEntry("RepeatDelay", 250);
+		double rate_ = config->readDoubleNumEntry("RepeatRate", 30);
+		set_repeatrate(delay_, rate_);
+	}
+
+
+	int numlockState = config->readNumEntry( "NumLock", 2 );
+	if( numlockState != 2 )
+		numlockx_change_numlock_state( numlockState == 0 );
+
+	delete config;
+}
+
 #include "kcmmisc.moc"
 
